#ifndef CHROMEOS_PLATFORMS_CAMERA_LIBHDR_GCAM_AE_H_
#define CHROMEOS_PLATFORMS_CAMERA_LIBHDR_GCAM_AE_H_

#include <cstdint>
#include <memory>
#include <vector>

#if defined(CHROME_OS_CAMERA_BUILD_ENV)
#include "cros-camera/ae_stats_intel_ipu6.h"
#else
#include "chromeos/platforms/camera/libhdr/ae_stats_intel_ipu6.h"
#endif

namespace cros {

// A normalized rectangle for a image with top-left (x0, y0) and bottom-right
// (x1, y1) coordinates in [0, 1].
struct NormalizedRect {
  float x0 = 0.0f;
  float x1 = 1.0f;
  float y0 = 0.0f;
  float y1 = 1.0f;

  float width() const { return x1 - x0; }
  float height() const { return y1 - y0; }
};

// AeResult carries the total exposure times (TETs) produced by the Gcam
// auto-exposure algorithm.
struct AeResult {
  // `short_tet` is the actual TET that should be applied to the sensor. Usually
  // we shorten the exposure time to avoid clipping the highlight areas.
  float short_tet = 0.0f;
  // `long_tet` is the desired exposure time for the shadow areas on the image,
  // and is used to produce the HDR ratio for the HDRnet pipeline.
  float long_tet = 0.0f;

  bool IsValid() const { return short_tet > 0.0f && long_tet > 0.0f; }
};

// AeFrameMetadata is a light-weight replica of
// gcam::cameralite::AeFrameMetadata and carries the input parameters to the
// Gcam AE algorithm.
struct AeFrameMetadata {
  float actual_analog_gain = 0.0f;
  float applied_digital_gain = 0.0f;
  float actual_exposure_time_ms = 0.0f;
  float sensor_sensitivity = 0.0f;
  NormalizedRect crop;
  std::vector<NormalizedRect> faces;
  std::vector<NormalizedRect> metering_areas;
};

// AWB and color correction metadata.
struct AwbInfo {
  // (R, Gr, Gb, B) color channel gains following the same definition as the
  // Android camera metadata 'ANDROID_COLOR_CORRECTION_GAINS'.
  float gains[4] = {0};

  // 3x3 color correction matrix following the same definition as the Android
  // camera metadata 'ANDROID_COLOR_CORRECTION_TRANSFORM'.
  float ccm[9] = {0};
};

// YuvFormat defines the YUV formats that can be accepted by the Gcam AE
// algorithm. Currently NV12 is the only supported format.
enum class YuvFormat {
  kNv12,
};

// YUV buffer metadata of a YUV buffer. Currently the only supported YUV format
// is NV12 hence `planes` has only size 2.
struct YuvBuffer {
  YuvFormat format;
  int width;
  int height;
  struct {
    int width;
    int height;
    int stride;
    const uint8_t* data;
  } planes[2];
};

class GcamAe {
 public:
  // Factory method to create a GcamAe instance.
  static std::unique_ptr<GcamAe> CreateInstance();

  // Computes a set of exposure parameters from the input `metadata`,
  // `yuv_buffer`, and `hdr_ratio`.  This is a somewhat generic and
  // device-agnostic approach that's good for initial evaluation of new
  // platforms.
  virtual AeResult ComputeLinearizedGcamAe(const AeFrameMetadata& metadata,
                                           const YuvBuffer& yuv_buffer,
                                           float max_hdr_ratio) = 0;

  // Computes exposure parameters through Gcam AE using Intel IPU6 AE stats.
  virtual AeResult ComputeGcamAe(int frame_width, int frame_height,
                                 const AeFrameMetadata& ae_metadata,
                                 const AwbInfo& awb_info,
                                 const AeStatsIntelIpu6& ae_stats_intel_ipu6,
                                 float max_hdr_ratio) = 0;

  virtual ~GcamAe() = default;
};

}  // namespace cros

#endif  // CHROMEOS_PLATFORMS_CAMERA_LIBHDR_GCAM_AE_H_
